@@ -18,7 +18,7 @@
 #' @param tail.p Optional \code{numeric} indicating the percentile of the simulated progeny to be included into the calculation of \eqn{\mu}\emph{_sp} and correlated response. Default is \code{0.10}.
 #' @param nInd Optional \code{integer} indicating the number of progeny simulated per cross, per iteration, using \code{\link[qtl]{sim.cross}} in R/qtl (\emph{Broman et al., 2003}). Default is \code{200}.
 #' @param map.plot Optional \code{logical}. If \code{TRUE} then a plot of the genetic map will be generated by \code{\link[qtl]{plot.map}}. Default is \code{FALSE}.
-#' @param impute Options include \code{c("EM", "mean", "pass")}. By default (i.e. \code{"EM"}), after filtering missing genotypic data will be imputed via the EM algorithm implemented in \code{\link{rrBLUP}} (\cite{Endelman, 2011}; \cite{Poland et al., 2012}). If \code{"mean"} missing genotypic data will be imputed via the 'marker mean' method, also implemented in \code{\link{rrBLUP}}. Enter \code{"pass"} if a pre-filtered and imputed genotype matrix is provided to \code{G.in}.
+#' @param impute Options include \code{c("EM", "mean", "pass")}. By default (i.e. \code{"EM"}), after filtering missing genotypic data will be imputed via the EM algorithm implemented in \code{\link{rrBLUP-package}} (\cite{Endelman, 2011}; \cite{Poland et al., 2012}). If \code{"mean"} missing genotypic data will be imputed via the 'marker mean' method, also implemented in \code{\link{rrBLUP-package}}. Enter \code{"pass"} if a pre-filtered and imputed genotype matrix is provided to \code{G.in}.
 #' @param min.maf Optional \code{numeric} indicating a minimum minor allele frequency (MAF) when filtering \code{G.in}. Markers with an MAF < \code{min.maf} will be removed. Default is \code{0.01} to remove monomorphic markers. Set to \code{0} for no filtering.
 #' @param mkr.cutoff Optional \code{numeric} indicating the maximum missing data per marker when filtering \code{G.in}. Markers missing > \code{mkr.cutoff} data will be removed. Default is \code{0.50}. Set to \code{1} for no filtering.
 #' @param entry.cutoff Optional \code{numeric} indicating the maximum missing genotypic data per entry allowed when filtering \code{G.in}. Entries missing > \code{entry.cutoff} marker data will be removed. Default is \code{0.50}. Set to \code{1} for no filtering.
@@ -31,13 +31,16 @@
 #' @param nIter,burnIn Optional \code{integer} arguments used by \code{\link[BGLR]{BGLR}} (\cite{de los Compos and Rodriguez, 2014}) when fitting Bayesian models to estimate marker effects. The defaults are \code{12000} and \code{3000}, respectively. These values when conducting CV are fixed \code{1500} and \code{500}, respectively, for computational efficiency.
 #' @param models Optional \code{Character vector} of the regression models to be used in CV and to estimate marker effects. Options include \code{rrBLUP, BayesA, BayesB, BayesC, BL, BRR}, one or more may be included at a time. CV will be conducted regardless of how many models are included. By default all models are tested.
 #' @param return.raw Optional \code{logical}. If \code{TRUE} then \code{pop.predict} will return the results of each simulation in addition to the summarized dataframe. Default is \code{FALSE}.
+#' @param saveAt When using models other than "rrBLUP" (i.e. Bayesian models), this is a path and prefix for saving temporary files 
+#' the are produced by the \code{\link[BGLR]{BGLR}} function.
+#' 
 #' @details \code{pop.predict} can be used to predict the mean (\eqn{\mu}), genetic variance (\emph{V_G}), superior progeny values (\eqn{\mu}\eqn{_sp}), as well as the predicted correlated response and correlations between all pairwise traits. The methodology and procedure to do so has been described in \cite{Bernardo (2014)} and \cite{Mohammadi, Tiede, and K.P. Smith (2015)}. Users familiar with genome-wide prediction, association mapping, and/or linkage mapping will be familiar with the
 #'          required inputs of \code{pop.predict}. \code{G.in} includes all of the entries (taxa) in the TP as well as additional entries to be considered as parent candidates. Entries included in \code{G.in} that do have a phenotype for any or all traits in \code{y.in} are considered TP entries for those respective traits. \code{G.in} is filtered according to \code{min.maf}, \code{mkr.cutoff}, \code{entry.cutoff}, and \code{remove.dups};
-#'          remaining missing marker data is imputed using the EM algorithm (\cite{Poland et al., 2012}) when possible, and the marker mean otherwise, both implemented in \code{\link{rrBLUP}}. For each trait, the TP (i.e. entries with phenotype) is used to: \enumerate{
+#'          remaining missing marker data is imputed using the EM algorithm (\cite{Poland et al., 2012}) when possible, and the marker mean otherwise, both implemented in \code{\link{rrBLUP-package}}. For each trait, the TP (i.e. entries with phenotype) is used to: \enumerate{
 #'          \item Perform CV to select a regression model. NOTE - Using the model with the highest CV accuracy is expected to result in the most accurate marker effect estimates (\cite{Bernardo, 2014}). This expectation, however, is yet to be empirically validated and the user is encouraged to investigate the various models in order to make an educated decision about which one to ultimately use.
 #'          \item Estimate marker effects using the model resulting in the highest CV accuracy
 #'          }
-#'          Models include ridge regression BLUP implemented in \code{\link{rrBLUP}} (\cite{Endelman, 2011}) and BayesA, BayesB, BayesC\eqn{\pi}, Bayesian lasso (BL), and Bayesian ridge regression (BRR) implemented in \code{\link{BGLR}} (\cite{de los Compos and Rodriguez, 2014}).
+#'          Models include ridge regression BLUP implemented in \code{\link{rrBLUP-package}} (\cite{Endelman, 2011}) and BayesA, BayesB, BayesC\eqn{\pi}, Bayesian lasso (BL), and Bayesian ridge regression (BRR) implemented in \code{\link{BGLR}} (\cite{de los Compos and Rodriguez, 2014}).
 #'          Information from the \code{map.in} is then used to simulate chromosomal recombination expected in a recombinant inbred line (i.e. \emph{F-infinity}) (\cite{Broman et al., 2003}) population (size=\code{nInd}). A function then converts the recombined chromosomal segments of the generic RIL population to the chromosomal segments of the population's respective parents and GEBVs of the simulated progeny are calculated.
 #'          The simulation and conversion process is repeated \emph{s} times, where \emph{s} = \code{nSim}, to calculate dispersion statistics for \eqn{\mu} and \emph{V_G}; the remainder of the values in the \code{predictions} output are means of the \emph{s} simulations.  During each iteration the correlation (\emph{r}) and correlated response of each pairwise combination of traits is also calculated and their mean across \emph{n} simulations is returned.
 #'          The correlated response of trait.B when predicting trait.A is the mean of trait.B for the (\eqn{\mu}\eqn{_sp}) of trait.A, and vice-versa; a correlated response for the bottom \code{tail.p} and upper \eqn{1-tail.p} is returned for each trait.
@@ -67,32 +70,23 @@
 #'      Poland, J., J. Endelman, J. Dawson, J. Rutkoski, S. Wu, Y. Manes, S. Dreisigacker, J. Crossa, H. Sanches-Villeda, M. Sorrells, and J.-L. Jannink. 2012. Genomic Selection in Wheat Breeding using Genotyping-by-Sequencing. Plant Genome 5:103-113.
 #'      
 #' @examples 
+#' 
 #' \dontrun{
-#' ## View formatting
-#' ## Use View() in RStudio or R GUI with X11 forwarding
-#' ## Use head() in R GUI without X11 forwarding
-#' View(G.in_ex)
-#' View(y.in_ex)
-#' View(map.in_ex)
-#' View(cross.tab_ex)
 #' 
-#' ## setwd() - pop.predict writes and reads files to disk
-#' ##   so it is recommended to set your working directory
-#' 
-#' ## nSim and nFold are set to low values in the
-#' ## examples for sake of computing time
+#' # Load data
+#' data("think_barley")
 #' 
 #' ## Ex. 1 - Predict a defined set of crosses
 #' ## This example uses CV method 1 (see Details of x.val() function)
 #' ex1.out <- pop.predict(G.in = G.in_ex, y.in = y.in_ex, 
 #'    map.in = map.in_ex, crossing.table = cross.tab_ex,
-#'    nSim=5, nCV.iter=10)
+#'    nSim=5, nCV.iter=2)
 #' ex1.out$predictions  ## Predicted parameters
 #' ex1.out$CVs          ## CV results
 #'                
 #' ## Ex. 2 - Predict all pairwise crosses between a list of parents
 #' ## This example uses CV method 2 (see Details of x.val() function)
-#' par.list <- sample(y.in_ex[,1], size = 10, replace = F)
+#' par.list <- sample(y.in_ex[,1], size = 10, replace = FALSE)
 #' ex2.out <- pop.predict(G.in = G.in_ex, y.in = y.in_ex,
 #'    map.in = map.in_ex, parents = par.list, 
 #'    nSim=5, nFold=5, nFold.reps=2)
@@ -101,6 +95,11 @@
 #' ex3.out <- pop.predict(G.in = G.in_ex, y.in = y.in_ex,
 #'    map.in = map.in_ex, crossing.table = cross.tab_ex,
 #'    models = c("rrBLUP", "BL"), nSim=5, nCV.iter=10)  
+#' 
+#' ## Ex. 4 - Same as Ex. 3, but return all raw SNP and prediction data for each simulated population
+#' ex4.out <- pop.predict(G.in = G.in_ex, y.in = y.in_ex,
+#'    map.in = map.in_ex, crossing.table = cross.tab_ex,
+#'    models = c("rrBLUP", "BL"), nSim=5, nCV.iter=2, return.raw = TRUE)  
 #' }
 #' 
 #' @importFrom utils write.table read.csv capture.output setTxtProgressBar txtProgressBar
@@ -109,11 +108,25 @@
 #' @export
 #' 
 pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL, 
-                        parents=NULL, tail.p=0.10, nInd=200, map.plot=F, min.maf=0.01, 
-                        mkr.cutoff=0.50, entry.cutoff=0.50, remove.dups=T, impute="EM", 
+                        parents=NULL, tail.p=0.10, nInd=200, map.plot=FALSE, min.maf=0.01, 
+                        mkr.cutoff=0.50, entry.cutoff=0.50, remove.dups=TRUE, impute="EM", 
                         nSim=25, frac.train=0.60, nCV.iter=100, nFold=NULL, nFold.reps=1, 
                         nIter=12000, burnIn=3000, models=c("rrBLUP", "BayesA", "BayesB","BayesC", "BL", "BRR"), 
-                        return.raw=F){
+                        return.raw=FALSE, saveAt = tempdir()){
+      
+  ## Testing only ##
+                      
+  # G.in=NULL; y.in=NULL; map.in=NULL; crossing.table=NULL
+  # parents=NULL; tail.p=0.10; nInd=200; map.plot=F; min.maf=0.01 
+  # mkr.cutoff=0.50; entry.cutoff=0.50; remove.dups=T; impute="EM" 
+  # nSim=25; frac.train=0.60; nCV.iter=100; nFold=NULL; nFold.reps=1 
+  # nIter=12000; burnIn=3000; models=c("rrBLUP", "BayesA", "BayesB","BayesC", "BL", "BRR") 
+  # return.raw=F
+  # 
+  # G.in = G.in_ex; y.in = y.in_ex
+  # map.in = map.in_ex; crossing.table = cross.tab_ex; nInd = 1000; nSim = 1; nCV.iter = 1; models = "rrBLUP"; return.raw = TRUE
+  
+  ## Testing ends
   
   ## QC steps
   if(is.null(G.in)) stop("Must provide a genotype (G.in) file.")
@@ -185,19 +198,17 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
   
   
   ### Read in genetic map for the markers
-  options(warn=-1) ## Temporarily turn off warnings
   read.map.out <-capture.output(read.map <- qtl::read.cross(format="csv", crosstype = "riself", file= paste("map.tmp_", name4out, ".csv", sep=""), na.strings="NA"))
-  print(paste("Number of Markers Read in: ", unlist(strsplit(read.map.out[3], split = " "), recursive = T)[2], sep = ""), quote=F)
+  print(paste("Number of Markers Read in: ", unlist(strsplit(read.map.out[3], split = " "), recursive = TRUE)[2], sep = ""), quote=FALSE)
   unlink(paste("map.tmp_", name4out, ".csv", sep=""))
   map_t1 <- qtl::pull.map(read.map)
-  options(warn=0)
-  if(map.plot==T) qtl::plot.map(map_t1)
+  if(map.plot) qtl::plot.map(map_t1)
   
   
   ## Imput missing markers with EM... will switch to imputing with the mean if nEntries > nMarkers
   ## Will need to use our own MAF filter so that we can keep track of which markers are removed due to MAF and missing data
-  if(impute == "EM") G.imp <- rrBLUP::A.mat(G.mat, min.MAF = 0, max.missing = 1, impute.method = "EM", return.imputed = T)$imputed
-  if(impute == "mean") G.imp <- rrBLUP::A.mat(G.mat, min.MAF = 0, max.missing = 1, impute.method = "mean", return.imputed = T)$imputed
+  if(impute == "EM") G.imp <- rrBLUP::A.mat(G.mat, min.MAF = 0, max.missing = 1, impute.method = "EM", return.imputed = TRUE)$imputed
+  if(impute == "mean") G.imp <- rrBLUP::A.mat(G.mat, min.MAF = 0, max.missing = 1, impute.method = "mean", return.imputed = TRUE)$imputed
   if(impute == "pass") G.imp <- G.mat
   
   ### Start simulation
@@ -214,22 +225,17 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
     G_TP <- G.imp[y_notNAs, ] 
     
     if(t==1){
-      cat("\n")
-      cat("Warnings about 'closing unused connections' AND 'Error in rinvGauss' can be safely disregarded... They are dealt with internally")
-      cat("\n")
+      message("Warnings about 'closing unused connections' AND 'Error in rinvGauss' can be safely disregarded... They are dealt with internally\n")
     }
     
-    cat("\n")
-    cat(paste("Selecting best model via cross validation for ", trait, " and estimating marker effects", sep=""))
-    cat("\n")    
+    message(paste("Selecting best model via cross validation for ", trait, " and estimating marker effects...\n", sep=""))
     
     if(is.null(nFold)) junk <- capture.output(xval.out <- XValidate_nonInd(y.CV = y_TP, G.CV = G_TP, models.CV = models, frac.train.CV=frac.train, nCV.iter.CV=nCV.iter, burnIn.CV = 750, nIter.CV = 1500)$CV.summary)
     if(!is.null(nFold)) junk <- capture.output(xval.out <- XValidate_Ind(y.CV = y_TP, G.CV = G_TP, models.CV = models, nFold.CV = nFold, nFold.CV.reps = nFold.reps, burnIn.CV = 750, nIter.CV = 1500)$CV.summary)
     
     if(t == 1){
       CV.results <- list()
-      mkr_effs.mat <- matrix(NA, ncol = nTraits, nrow = ncol(G.imp))
-      colnames(mkr_effs.mat) <- traits
+      mkr_effs.mat <- matrix(NA, ncol = nTraits, nrow = ncol(G.imp), dimnames = list(colnames(G.imp), traits))
       beta.list <- c()
       best.models <- c()
     }
@@ -245,7 +251,8 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
       beta <- as.numeric(mix.solve.out$beta)
       mkr_effects <- mix.solve.out$u
     }else{
-      capture.output(bayes.fit <- BGLR::BGLR(y=y_TP, ETA=list(list(X=G_TP, model=best.model)), verbose=F, nIter=nIter, burnIn=burnIn))
+      capture.output(bayes.fit <- BGLR::BGLR(y=y_TP, ETA=list(list(X=G_TP, model=best.model)), verbose=FALSE, nIter=nIter, 
+                                             burnIn=burnIn, saveAt = saveAt))
       mkr_effects <- as.numeric(bayes.fit$ETA[[1]]$b)
       beta <- bayes.fit$mu  
     }
@@ -255,8 +262,7 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
     
   }
   
-  cat("\n")
-  cat("Cross validation is complete!")
+  message("Cross validation is complete!")
   
   ## Set up which crosses to predict
   if(!is.null(crossing.table)){ ## Used when the user provides a list of specific crosses, parents come from G.in.entries (i.e. not necessarily in TP)
@@ -303,17 +309,19 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
   
   
   ## Start simulation and var.prd process
-  cat("\n")
-  cat(paste("\nBrewing", nSim, "populations of", nInd, "individuals for each cross... Please be patient", sep=" "))
-  cat("\n")
+  message(paste("\nBrewing", nSim, "populations of", nInd, "individuals for each cross... Please be patient\n", sep=" "))
+  # Create an empty list for storing the progeny SNP genotypes and gebvs
+  raw_results_per_cross <- lapply(X = seq_len(nrow(crossing.mat)), FUN = function(i) {
+    `names<-`(vector("list", nSim), paste0("sim", formatC(x = seq_len(nSim), width = nchar(nSim), flag = "0")))
+  })
   prog.bar <- txtProgressBar(min=1,  max=(nrow(crossing.mat)*nSim), style=3); p=1
   M <- nInd
   
   for(s in 1:nSim){
     sim.pop <- qtl::sim.cross(map_t1, type="riself", n.ind = M, model=NULL)
     qtl::write.cross(sim.pop, "csv", paste("sim.pop.tmp_", name4out, sep="")) ## NEED to figure out a way to not read it out and in
-    pop.mat <- as.matrix(read.csv(paste("sim.pop.tmp_", name4out, ".csv", sep=""), header=T))[3:(M+2), 2:(length(mkr_effects)+1)]
-    unlink(paste("sim.pop.tmp_", name4out, ".csv", sep=""))   
+    pop.mat <- as.matrix(read.csv(paste("sim.pop.tmp_", name4out, ".csv", sep=""), header=TRUE))[3:(M+2), 2:(length(mkr_effects)+1)]
+    unlink(paste("sim.pop.tmp_", name4out, ".csv", sep=""))  
     
     for(z in 1:nrow(crossing.mat)){
       setTxtProgressBar(prog.bar, p)
@@ -333,16 +341,27 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
       
       if(ncol(replace.0.mat) > 0){
         for(b in 1:ncol(replace.0.mat)){
-          pop.mat2[which(pop.mat2[,replace.0.mat[1,b]] == 0), replace.0.mat[1,b]] <- sample(c(1,-1), size = replace.0.mat[2,b],  replace = T, prob = c(.5,.5))
+          pop.mat2[which(pop.mat2[,replace.0.mat[1,b]] == 0), replace.0.mat[1,b]] <- sample(c(1,-1), size = replace.0.mat[2,b],  replace = TRUE, prob = c(.5,.5))
         }
       }
       
+      # Add names to the pop mat
+      dimnames(pop.mat2) <- list(paste0("RIL", formatC(seq_len(nInd), width = nchar(nInd), flag = "0")),
+                                 row.names(mkr_effs.mat))
+      
+      # Calculage GEBVs
       prog_pred.mat <- pop.mat2 %*% mkr_effs.mat
+
       for(b in 1:length(beta.list)){
         beta.tmp <- beta.list[b]
         prog_pred.mat[,b] <- prog_pred.mat[,b] +  beta.tmp
       }
       
+      
+      # Add SNPs and predicted values to the cross_results list - if return.raw is true
+      if (return.raw) raw_results_per_cross[[z]][[s]] <- list(snps = pop.mat2, gebvs = prog_pred.mat)
+      
+      
       for(n in 1:nTraits){
         if(s == 1){
           if(nTraits > 1) colnames(param.dfs[[n]])[11:(10+3*(nTraits-1))]<- c(paste("low.resp_", traits[-n], sep=""), paste("high.resp_", traits[-n], sep=""), paste("cor_w/_", traits[-n], sep=""))
@@ -358,8 +377,8 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
         ## Calculate correlations between traits and correlated response and 
         if(nTraits > 1){
           index <- 1; for(n2 in (1:nTraits)[-n]){
-            param.dfs[[n]][[z, 10+index]][s] <- mean(prog_pred.mat[,n2][which(prog_pred.mat[,n] <= quantile(prog_pred.mat[,n], probs = tail.p))], na.rm = T)
-            param.dfs[[n]][[z, 10+(nTraits-1)+index]][s] <- mean(prog_pred.mat[,n2][which(prog_pred.mat[,n] >= quantile(prog_pred.mat[,n], probs = 1-tail.p))], na.rm = T)
+            param.dfs[[n]][[z, 10+index]][s] <- mean(prog_pred.mat[,n2][which(prog_pred.mat[,n] <= quantile(prog_pred.mat[,n], probs = tail.p))], na.rm = TRUE)
+            param.dfs[[n]][[z, 10+(nTraits-1)+index]][s] <- mean(prog_pred.mat[,n2][which(prog_pred.mat[,n] >= quantile(prog_pred.mat[,n], probs = 1-tail.p))], na.rm = TRUE)
             param.dfs[[n]][[z, 10+2*(nTraits-1)+index]][s] <- cor(prog_pred.mat[,n], prog_pred.mat[,n2], use = "complete.obs")
             index <- index+1
           }
@@ -368,23 +387,39 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
       
       p <- p+1 # This is the counter for the progress bar
     } ## End of z loop for each cross
+    
+    
   }## End of nSim (s) loop
   
-  preds.per.sim <- param.dfs
-  
   for(n in 1:nTraits){
     col.names <- colnames(param.dfs[[n]])
     for(c in 3:length(col.names)){
       name.tmp <- col.names[c]
-      if(name.tmp %in% c("pred.mu_sd", "pred.varG_sd")) param.dfs[[n]][,c] <- sapply(param.dfs[[n]][,c], FUN = sd, na.rm=T)
-      if(!name.tmp %in% c("pred.mu_sd", "pred.varG_sd")) param.dfs[[n]][,c] <- sapply(param.dfs[[n]][,c], FUN = mean, na.rm=T)
+      if(name.tmp %in% c("pred.mu_sd", "pred.varG_sd")) param.dfs[[n]][,c] <- sapply(param.dfs[[n]][,c], FUN = sd, na.rm=TRUE)
+      if(!name.tmp %in% c("pred.mu_sd", "pred.varG_sd")) param.dfs[[n]][,c] <- sapply(param.dfs[[n]][,c], FUN = mean, na.rm=TRUE)
     }
   }
   
   if(nTraits == 1) param.dfs <- as.data.frame(param.dfs[[1]])
   
-  if(return.raw) return(list(predictions=param.dfs, preds.per.sim=preds.per.sim, CVs=CV.results, models.chosen=best.models, markers.removed=mkrs.removed, entries.removed=entries.removed))
-  if(!return.raw) return(list(predictions=param.dfs, CVs=CV.results, models.chosen=best.models, markers.removed=mkrs.removed, entries.removed=entries.removed))
+  # Flag based on return.raw
+  if (return.raw) {
+    return(list(predictions=param.dfs, 
+                preds.per.sim=raw_results_per_cross, 
+                CVs=CV.results, 
+                models.chosen=best.models, 
+                markers.removed=mkrs.removed, 
+                entries.removed=entries.removed))
+    
+  } else {
+    return(list(predictions=param.dfs, 
+                CVs=CV.results, 
+                models.chosen=best.models, 
+                markers.removed=mkrs.removed, 
+                entries.removed=entries.removed))
+    
+  }
+
   
 } # End of pop.predict
 
@@ -414,9 +449,21 @@ pop.predict <- function(G.in=NULL, y.in=NULL, map.in=NULL, crossing.table=NULL,
 #' Predictions are based on the deterministic equations specified by Zhong and Jannink (2007), Allier et al. (2019),
 #' and Neyhart et al. (2019).
 #' 
+#' If you select a \code{model} other than "rrBLUP", you must specify the following additional arguments:
+#' \itemize{
+#'   \item{\code{nIter}: See \code{\link[PopVar]{pop.predict}}}.
+#'   \item{\code{burnIn}: See \code{\link[PopVar]{pop.predict}}}.
+#' }
+#' 
+#' 
+#' @return
+#' A \code{data.frame} containing predictions of \eqn{\mu}, \eqn{V_G}, and \eqn{\mu_{sp}} for 
+#' each trait for each potential bi-parental cross. When multiple traits are provided, the correlated 
+#' responses and correlation between all pairs of traits is also returned.
+#' 
 #' @examples 
 #' 
-#' \dontrun{
+#' \donttest{
 #' 
 #' # Load data
 #' data("think_barley")
@@ -527,10 +574,10 @@ pop.predict2 <- function(G.in, y.in, map.in, crossing.table, parents, tail.p = 0
     
   }
   
-  if (any(!parents %in% G.in_use[,1,drop = T]))
+  if (any(!parents %in% G.in_use[,1,drop = TRUE]))
     stop("Parents are not in G.in.")
   
-  ## If self.gen is Inf and DH is T, error
+  ## If self.gen is Inf and DH is TRUE, error
   if (is.infinite(self.gen) & DH) stop("Infinite selfing generations and doubled-haploid production cannot both occur.")
   
   ## Set the factors of line names in y.in to those in the marker df and those in the y.in
@@ -577,6 +624,8 @@ pop.predict2 <- function(G.in, y.in, map.in, crossing.table, parents, tail.p = 0
 #'
 #' @examples
 #' 
+#' \donttest{
+#' 
 #' # Load data
 #' data("think_barley")
 #' 
@@ -589,17 +638,8 @@ pop.predict2 <- function(G.in, y.in, map.in, crossing.table, parents, tail.p = 0
 #' out <- pop_predict2(M = G.in_ex_mat, y.in = y.in_ex, map.in = map.in_ex, 
 #'                     parents = y.in_ex$Entry[1:10])
 #'                     
-#' # Make predictions for 5 crosses with various levels of inbreeding
-#' out_list <- lapply(X = 1:10, FUN = function(self.gen) {
-#'   out <- pop_predict2(M = G.in_ex_mat, y.in = y.in_ex, map.in = map.in_ex, 
-#'                       crossing.table = cross.tab_ex[1:5,], self.gen = self.gen)
-#'   out$self.gen <- self.gen
-#'   out })
-#'                
-#' # Plot predictions of grain yield genetic variance over levels of inbreeding
-#' dat <- do.call("rbind", lapply(out_list, subset, trait == "Yield"))
-#' plot(pred_varG ~ self.gen, data = dat, type = "b", 
-#'      subset = parent1 == parent1[1] & parent2 == parent2[1])
+#' }
+#'                     
 #'
 #' @importFrom qtl mf.h
 #' @importFrom rrBLUP mixed.solve
